class: Workflow
cwlVersion: v1.1
id: compress_pipeline_cwl
label: compress_pipeline.cwl
inputs:
  - id: flag_autocorrelation
    type: boolean?
  - id: msin
    type: Directory
outputs:
  - id: inspect
    outputSource:
      - inspect_flagging_dataloss/output
    type: 'File'
  - id: logfile
    outputSource:
      - dppp/logfile
    type: File[]
  - id: compressed
    outputSource:
      - compress/compressed
    type: File
  - id: ingest
    outputSource:
      - format_ingest/ingest
    type: Any
  - id: uv_plot
    type: File
    outputSource:
      - plot_uvw_coverage/uv_plot
  - id: uv_coverage
    type: File
    outputSource:
      - inspect_flagging_dataloss/flags_output
steps:
  - id: extract_sip_meta
    in:
      - id: msin
        source: dppp/msout
      - id: compressed_file
        source: compress/compressed
    out:
      - id: ingest
    run: steps/extract_sip_meta.cwl
  - id: dppp
    in:
      - id: parset
        source: define_parset/output
      - id: msin
        source: msin
      - id: msout_name
        source: msin
        valueFrom: '$("COMPRESSED_" + self.basename)'
      - id: writefullresflag
        default: true
      - id: storagemanager
        default: Dysco
      - id: databitrate
        default: 10
    out:
      - id: msout
      - id: logfile
    run: steps/DPPP.cwl
  - id: format_ingest
    in:
      - id: metadata
        source: extract_sip_meta/ingest
      - id: output_name
        default: compressed
    out:
      - id: ingest
    run: steps/format_ingest.cwl
  - id: define_parset
    in:
      - id: flag_autocorrelation
        source: flag_autocorrelation
    out:
      - id: output
    run: steps/define_parset.cwl
    label: define_parset
  - id: inspect_flagging_dataloss
    in:
      - id: input
        source: msin
    out:
      - id: output
      - id: flags_output
    run: steps/inspect_flagging_dataloss.cwl
    label: inspect_flagging_dataloss
  - id: plot_uvw_coverage
    in:
      - id: input_file
        source: inspect_flagging_dataloss/flags_output
    out:
      - id: uv_plot
    run: steps/plot_uvw_coverage.cwl
  - id: compress
    in:
      - id: directory
        source: dppp/msout
    out:
      - id: compressed
    run: steps/compress.cwl
    label: compress
requirements:
  - class: StepInputExpressionRequirement
  - class: InlineJavascriptRequirement
